package de.hdm_stuttgart.mi.sda1;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.*;
import org.jdom2.input.SAXBuilder;

import java.io.File;
import java.io.IOException;
import java.sql.*;

public class Examination2Rdbms implements AutoCloseable {

  static private final Logger log = LogManager.getLogger(Examination2Rdbms.class);

  static private final String
          insertLectureSql = "INSERT INTO Lecture (id, title) VALUES (?,  ?)",

  insertMarkSql =
      "INSERT INTO  Mark VALUES (?, ?, ?, ?)";

  private Connection conn;

  private Element examinations;

  private PreparedStatement insertLecture, insertMark;

  public Examination2Rdbms() {
    try {
      conn = DriverManager.getConnection(
          "jdbc:postgresql://localhost/hdm", "hdmuser", "XYZ");
      insertLecture = conn.prepareStatement(insertLectureSql);
      insertMark = conn.prepareStatement(insertMarkSql);
    } catch (final SQLException e) {
      log.error("Unable to establish connection, exiting", e);
      System.exit(1);
    }

    final SAXBuilder builder = new SAXBuilder();
    try {
      examinations = builder.build(new File("Schema/sampledata.xml")).getRootElement();
    } catch (final JDOMException e) {
      log.error("Unable to parse input, exiting", e);
      System.exit(1);
    } catch (final IOException e) {
      log.error("Unable to read input, exiting", e);
      System.exit(1);
    }
  }

  public void execute() {
    log.info("Parsing XML input file");
    examinations.getChild("lectures").getChildren("lecture").forEach(this::handleLecture);
    examinations.getChild("examinationResults").getChildren("result").forEach(this::handleResult);
  }

  private void handleLecture(final Element lecture) {
    log.info("Try inserting Lecture ");
    final String
        id = lecture.getAttributeValue("id"),
        title = lecture.getValue();
    log.info("Lecture id = " + id);
    try {
      insertLecture.setString(1, id);
      insertLecture.setString(2, title);
      insertLecture.execute();
    } catch (final SQLIntegrityConstraintViolationException e) {
      log.info("Possibly existing lecture");
    } catch (final SQLException e) {
      log.error("Failed to insert category id = " + id + ": ", e);
    }
  }

  private void handleResult(final Element result) {
    log.info("Inserting mark");

    final int lectureId = Integer.parseInt(result.getAttributeValue("lecture"));
    final int matriculationNumber = Integer.parseInt(result.getAttributeValue("matriculationNumber"));
    final String status = result.getAttributeValue("status");
    final String mark = result.getValue();

    log.info("lecture = " + lectureId + ", matriculation = " + matriculationNumber);
    try {
      insertMark.setInt(1, matriculationNumber);
      insertMark.setInt(2, lectureId);
      insertMark.setString(3, mark);
      insertMark.setString(4, status);
      insertMark.execute();
    } catch (final SQLException e) {
      log.error("Failed to insert article orderNo = " + lectureId + ": ", e);
    }
  }

  @Override
  public void close() {
    try {
      conn.close();
    } catch (final SQLException e) {
      e.printStackTrace();
    }
  }
}